/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Toporov Konstantin.
 */

package org.diacalc.simple;

import com.sun.lwuit.events.*;
import com.sun.lwuit.*;
import com.sun.lwuit.util.Resources;
import com.sun.lwuit.plaf.UIManager;
import com.sun.lwuit.impl.midp.VKBImplementationFactory;
import com.sun.lwuit.layouts.*;
import java.util.Hashtable;
import com.sun.lwuit.animations.CommonTransitions;
import com.sun.lwuit.list.*;
import org.diacalc.mobile.products.*;
import org.diacalc.mobile.maths.*;
import com.sun.lwuit.plaf.Border;
import javax.microedition.rms.*;
import java.io.*;
import java.util.Vector;
import java.util.Enumeration;
import javax.microedition.io.file.FileConnection;
import javax.microedition.io.*;

/**
 *
 * @author Toporov Konstantin <www.diacalc.org>
 */

public class DCJsimple extends javax.microedition.midlet.MIDlet
        implements ActionListener{
    public final static String MEALFILE = "meal.txt";

    private static final int EXIT_COMMAND = 1;
    private static final int ABOUT_COMMAND = 2;
    private static final int IMPORT_COMMAND = 3;

    private static final Command exitCommand = new Command("Выход", EXIT_COMMAND);
    private static final Command aboutCommand = new Command("О программе", ABOUT_COMMAND);
    private static final Command importCommand = new Command("Импорт", IMPORT_COMMAND);
    

    private Form mainMenu;
    private Label fldBD;
    private Label fldMD;
    private Label fldSUM;
    private Label fldBEres;

    private TextField fldK1;
    private TextField fldK2;
    private TextField fldBE;
    
    private float k1 = 1f;
    private float k2 = 0f;
    private float be = 10f;

    private final static String DCJ_RMS = "DCJsimple";
    private RecordStore rs = null;
    
    private static int GROUPS = 6;

    private DefaultListModel [] models = null;
    private TextField [] weights = null;
    private Vector [] base = null;

    MathEvaluator evaluter = new MathEvaluator();
   
    public void startApp() {
        initProds();
        try {
            //By using the VKBImplementationFactory.init() we automatically
            //bundle the LWUIT Virtual Keyboard.
            //If your application is not aimed to touch screen devices,
            //this line of code should be removed.
            VKBImplementationFactory.init();
            Display.init(this);
            
            //set the theme
            Resources theme = Resources.open("/DCJtheme.res");
            UIManager.getInstance().setThemeProps(theme.getTheme(theme.getThemeResourceNames()[0]));
            //open the resources file that contains all the icons
            
            //although calling directly to setMainForm(r2) will work on
            //most devices, a good coding practice will be to allow the midp
            //thread to return and to do all the UI on the EDT.
            makeText();

            Display.getInstance().callSerially(new Runnable() {
                public void run() {
                    setMainForm();
                }
            });

        } catch (Throwable ex) {
            //ex.printStackTrace();
            Dialog.show("Exception", ex.getMessage(), "OK", null);
        }
    }

    public Vector fromByteArray( byte[] data )
                                 throws IOException {
        ByteArrayInputStream bin =
                 new ByteArrayInputStream( data );
        DataInputStream din = new DataInputStream( bin );

        int count = din.readInt();
        Vector v = new Vector( count );

        while( count-- > 0 ){
            v.addElement( new ProductW(
                    din.readUTF(),//name
                    din.readFloat(),//prot
                    din.readFloat(),//fat
                    din.readFloat(),//carb
                    din.readInt(),//gi
                    100f) );
        }

        din.close();

        return v;
    }
    public byte[] toByteArray( Vector v )
                               throws IOException {
        ByteArrayOutputStream bout =
                   new ByteArrayOutputStream();
        DataOutputStream dout = new DataOutputStream( bout );

        int sz = v.size();
        dout.writeInt( sz );

        Enumeration e = v.elements();
        while( e.hasMoreElements() ){
            ProductW prod = (ProductW)e.nextElement();
            dout.writeUTF(prod.getName());
            dout.writeFloat(prod.getProt());
            dout.writeFloat(prod.getFat());
            dout.writeFloat(prod.getCarb());
            dout.writeInt(prod.getGi());
        }

        dout.close();

        return bout.toByteArray();
    }

    private void initProds(){
        boolean defaultinit = true;
        try {
            rs = RecordStore.openRecordStore( DCJ_RMS, true );
            
            if (rs.getNumRecords()>0){
                //Есть записи о коэф-тах
                ByteArrayInputStream bin;
                DataInputStream din;

                byte[] data = rs.getRecord( 1 );
                try{
                    bin = new ByteArrayInputStream( data );
                    din = new DataInputStream( bin );

                    k1 = din.readFloat();
                    k2 = din.readFloat();
                    be = din.readFloat();
                    din.close();
                }
                catch( IOException e ){
                    //e.printStackTrace();
                }
            }
            if (rs.getNumRecords()>1){
                //Есть записи о группах продуктов, продолжаем
                ByteArrayInputStream bin;
                DataInputStream din;

                byte[] data = rs.getRecord( 2 );//Кол-во групп
                try{
                    bin = new ByteArrayInputStream( data );
                    din = new DataInputStream( bin );

                    GROUPS = din.readInt();
                    din.close();
                }
                catch( IOException e ){
                    //e.printStackTrace();
                }

                models = new DefaultListModel[GROUPS];
                weights = new TextField[GROUPS];
                base = new Vector[GROUPS];
                
                boolean failed = false;
                for(int i=0;i<GROUPS;i++){
                    try{
                        base[i] = fromByteArray(rs.getRecord( i+3 ));
                    }
                    catch(IOException ex){
                        //ex.printStackTrace();
                        failed = true;
                    }
                }
                if (!failed) defaultinit = false;//все удалось, используем загруженные данные
            }
        } catch( RecordStoreException e ){
            //e.printStackTrace();
        } finally {
            try {
                rs.closeRecordStore();
            } catch( RecordStoreException e ){
            // Ignore this exception
            }
        }

        if (defaultinit){
            GROUPS = 6;
            models = new DefaultListModel[GROUPS];
            weights = new TextField[GROUPS];
            base = new Vector[GROUPS];

            base[0] = new Vector();
            base[0].addElement(new ProductW("Лагман", 3.1f, 2.2f, 10.5f, 55, 100f));
            base[0].addElement(new ProductW("Грибной с картофелем", 1.4f, 2.0f, 2.4f, 48, 100f));

            base[1] = new Vector();
            base[1].addElement(new ProductW("Вареная гречка", 4.2f, 1.0f, 22.7f, 40, 100f));
            base[1].addElement(new ProductW("Вареные - макароны", 3.8f, 0.5f, 27.0f, 30, 100f));

            base[2] = new Vector();
            base[2].addElement(new ProductW("Бефстроганов (R)", 21.9f, 27.4f, 5.7f, 60, 100f));
            base[2].addElement(new ProductW("Бифштекс (R)", 27.8f, 29.5f, 1.7f, 60, 100f));

            base[3] = new Vector();
            base[3].addElement(new ProductW("Мясной салат", 5.7f, 16.0f, 5.7f, 53, 100f));
            base[3].addElement(new ProductW("Овощной с маслом", 1.9f, 6.2f, 3.4f, 30, 100f));

            base[4] = new Vector();
            base[4].addElement(new ProductW("Хлеб черный", 7.0f, 1.2f, 40.6f, 76, 100f));
            base[4].addElement(new ProductW("Хлеб белый", 7.6f, 0.9f, 46.7f, 88,100f));

            base[5] = new Vector();
            base[5].addElement(new ProductW("Сахар", 0f, 0f, 99.8f, 65, 100f));
            base[5].addElement(new ProductW("Сок томатный", 1f, 0f, 3.3f, 15,100f));
            base[5].addElement(new ProductW("Сок яблочный", 0.5f, 0.0f, 11.7f, 40, 100f));
            base[5].addElement(new ProductW("Чай Lipton", 0f, 0f, 6.9f, 70, 100f));
        }
    }
    public void actionPerformed(ActionEvent ev){
        Command cmd = ev.getCommand();
        if (cmd!=null)
        switch (cmd.getId()) {
            case EXIT_COMMAND:
                //System.out.println("Exit command");
                destroyApp(true);
                break;
            case ABOUT_COMMAND:
                Form aboutForm = new Form("О программе");
                aboutForm.setScrollable(false);
                aboutForm.setLayout(new BorderLayout());
                TextArea aboutText = new TextArea(getAboutText(), 5, 10);
                aboutText.setEditable(false);
                aboutForm.addComponent(BorderLayout.CENTER, aboutText);
                Command bkCm = new Command("Назад") {
                    public void actionPerformed(ActionEvent evt) {
                        mainMenu.show();
                    }
                };
                aboutForm.addCommand(bkCm);
                aboutForm.setBackCommand(bkCm);
                aboutForm.show();
                break;
            case IMPORT_COMMAND:
                try{
                    ExploreForm importForm = new ExploreForm("Импорт",mainMenu,this);
                    importForm.show();
                }catch (Exception ex){
                    Dialog.show("Ошибка!",
                            new TextArea("Ошибка доступа к файловой системе\n"+ex.getMessage(),3,20),
                            new Command[] {new Command("Ok")},
                            Dialog.TYPE_ERROR,
                            null);
                }
                break;
        }
        else  calcMenu();
    }
    private final static String numb = "1234567890.+-*/()";
    private final static String nodouble = ".+-*/";

    private float parseString(TextField row,int zeros){
        String st = row.getText().trim();
        if (st.length()==0) return 0f;
        String st1="";
        //удаляем все неиспользуемые символы
        for(int i=0;i<st.length();i++){
            for(int j=0;j<numb.length();j++){
                if (st.charAt(i)==numb.charAt(j)){
                    st1 += st.charAt(i);
                    break;
                }
            }
        }
        //Двойные символы так же надо убрать
        if (st1.length()>1){
            int i=0;
            boolean found = false;
            while (i<(st1.length()-1)){
                found = false;
                if (st1.charAt(i)==st1.charAt(i+1)){
                    for(int j=0;j<nodouble.length();j++){
                        if (st1.charAt(i)==nodouble.charAt(j)){
                            //Удаляем
                            st1 = st1.substring(0, i) + st1.substring(i+1);
                            found = true;
                            break;
                        }
                    }
                }
                if (!found) i++;
            }
        }
        
        //считаем скобки
        int c=0;
        int a=0;
        for(int i=0;i<st1.length();i++){
            if (st1.charAt(i)=='(') c++;
            if (st1.charAt(i)==')') a++;
        }
        if (c>a){//Добавляем в конец ) скобки
            for(int i=0;i<(c-a);i++){
                st1 += ')';
            }
        }
        else if (c<a){//Добавляем в начало ( скобки
            for(int i=0;i<(a-c);i++){
                st1 = '(' + st1;
            }
        }
        float res;
        evaluter.setExpression(st1);
        try{
            res = evaluter.getValue().floatValue();
        }
        catch(Exception ex){
            res = 0f;
        }
        res = res>0?res:0f;
        if (zeros==1) row.setText(""+round0(res));
        else if (zeros==2) row.setText(""+round00(res));
        else if (zeros==0) row.setText(""+round(res));
        else row.setText(""+res);
        return res;
    }
    private void saveData(boolean prods){
        try{
            rs = RecordStore.openRecordStore( DCJ_RMS, true );
            ByteArrayOutputStream bout = new ByteArrayOutputStream();
            DataOutputStream dout = new DataOutputStream( bout );

            dout.writeFloat( parseString(fldK1, 2) );
            dout.writeFloat( parseString(fldK2, 2) );
            dout.writeFloat( parseString(fldBE, 2) );
            dout.close();
            byte[] data = bout.toByteArray();
            if (rs.getNumRecords()==0){
                rs.addRecord(data, 0, data.length);
            }else{
                rs.setRecord(1, data, 0, data.length );
            }
            if (prods){
                bout = new ByteArrayOutputStream();
                dout = new DataOutputStream( bout );

                dout.writeInt(base.length);
                dout.close();

                data = bout.toByteArray();
                if (rs.getNumRecords()==1){
                    rs.addRecord(data, 0, data.length);
                }else{
                    rs.setRecord(2, data, 0, data.length );
                }//записали кол-во групп

                for(int i=0;i<base.length;i++){
                    data = toByteArray(base[i]);
                    if (rs.getNumRecords()==(2+i)){//Добавляем
                        rs.addRecord(data, 0, data.length);
                    }
                    else{//Используем
                        rs.setRecord(i+3,data, 0, data.length);
                    }
                    
                }
            }
        }
        catch( RecordStoreException e ){
            // handle RMS error here
            //e.printStackTrace();
        }
        catch( IOException e ){
            // handle IO error here
            //e.printStackTrace();
        }finally {
            try {
                rs.closeRecordStore();
            } catch( RecordStoreException e ){
                //e.printStackTrace();
            // Ignore this exception
            }
        }
    }
    
    private void calcMenu(){
        ProductW prod = new ProductW();
        for(int i=0;i<models.length;i++){
            ProductW p = (ProductW)models[i].getItemAt(models[i].getSelectedIndex());
            p.setWeight(parseString( weights[i],0 ));
            prod.plusProd(  p );
        }
        //Теперь парсим коэф-ты
        Factors fc = new Factors(
                parseString(fldK1, 2),
                parseString(fldK2, 2),
                0f,
                parseString(fldBE, 2));
        Dose ds = new Dose(prod,fc,new DPS());
        fldBD.setText(""+round0(ds.getCarbFastDose()));
        fldMD.setText(""+round0(ds.getCarbSlowDose()+ds.getSlowDose()));
        fldSUM.setText(""+round0(ds.getWholeDose()));
        fldBEres.setText(""+round0(prod.getAllCarb()/fc.getBE(Factors.DIRECT)));
    }
    private void setMainForm(){
        mainMenu = new Form("DCJ simple");
        mainMenu.setTransitionOutAnimator(CommonTransitions.createFade(400));
        mainMenu.setLayout(new BorderLayout());

        mainMenu.addComponent(BorderLayout.NORTH, createTop());

        Container list = new Container(new BoxLayout(BoxLayout.Y_AXIS));
        
        for(int i=0;i<models.length;i++){
            list.addComponent(createRow(i));
        }
        list.addComponent(createCoefPane());
        list.setScrollableY(true);
        
        mainMenu.addComponent(BorderLayout.CENTER, list);

        mainMenu.addCommand(exitCommand);
        mainMenu.addCommand(importCommand);
        mainMenu.addCommand(aboutCommand);

        mainMenu.addCommandListener(this);
        mainMenu.setScrollableY(false);
        mainMenu.show();
        
        fldK1.setText(""+k1);
        fldK2.setText(""+k2);
        fldBE.setText(""+be);
    }
    private Container createRow(int row){
        Container con = new Container();

        models[row] = new DefaultListModel(base[row]);
        ComboBox prods = new ComboBox(models[row]){ protected List createPopupList() {
                  List l = super.createPopupList();
                  final TickerRenderer trndr = new TickerRenderer();
                  l.setListCellRenderer(trndr);
                  l.addSelectionListener(new SelectionListener(){
                    public void selectionChanged(int o,int n){
                        if (o!=n){
                            trndr.restartTicker();
                        }
                    }
                  });
                  return l;
              }
        };
        
        prods.addActionListener(this);

        Label lbl = new Label("Вес");
        weights[row] = new TextField();
        weights[row].setConstraint(TextField.DECIMAL);
        weights[row].addActionListener(this);

        GroupLayout layout = new GroupLayout(con);
        con.setLayout(layout);
        
        layout.setHorizontalGroup(layout.createParallelGroup()
                .add(prods, 0, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(layout.createSequentialGroup()
                    .add(lbl)
                    .add(weights[row], 0, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                );
        layout.setVerticalGroup(layout.createSequentialGroup()
                .add(prods)
                .add(layout.createParallelGroup()
                    .add(lbl)
                    .add(weights[row])
                 ));
        return con;
    }
    private int round(float v){
       return (int)(v+0.5f);
    }
    private float round0(float v){
       return (float)(int)((v+0.05f)*10.0f)/10.0f;
    }
    private float round00(float v){
       //round to 2 decimal points
       //double number = (double)(int)((bmi+0.005)*100.0)/100.0;
       //round to 4 decimal points
       //double number = (double)(int)((bmi+0.00005)*10000.0)/10000.0;
       return (float)(int)((v+0.005f)*100.0f)/100.0f;
    }
    private Label createTxtField(){
        Label fld = new Label("===");
        fld.setAlignment(TextField.CENTER);
        fld.getStyle().setBorder(Border.createBevelLowered());
        return fld;
    }
    private Container createTop(){
        Container con = new Container();
        Label lblBD = new Label("Б");
        Label lblMD = new Label("М");
        Label lblSUM = new Label("С");

        Label lblBE = new Label("XE");

        fldBD = createTxtField();
        fldMD = createTxtField();
        fldSUM = createTxtField();
        fldBEres = createTxtField();

        GroupLayout layout = new GroupLayout(con);
        con.setLayout(layout);

        layout.setAutocreateGaps(true);

        layout.setHorizontalGroup(layout.createParallelGroup()
                .add(layout.createSequentialGroup()
                    .add(lblBD)
                    .add(fldBD, 0, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(lblMD)
                    .add(fldMD, 0, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .add(layout.createSequentialGroup()
                    .add(lblSUM)
                    .add(fldSUM, 0, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(lblBE)
                    .add(fldBEres, 0, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    )
                );
        layout.setVerticalGroup(layout.createSequentialGroup()
                .add(layout.createParallelGroup()
                    .add(lblBD)
                    .add(fldBD)
                    .add(lblMD)
                    .add(fldMD))
                .add(layout.createParallelGroup()
                    .add(lblSUM)
                    .add(fldSUM)
                    .add(lblBE)
                    .add(fldBEres))
                );
        con.getStyle().setBorder(Border.createEtchedRaised());
        return con;
    }
    private Container createCoefPane(){
        Container con = new Container();
        Label lblK1 = new Label("k1");
        fldK1 = new TextField();
        fldK1.setConstraint(TextField.DECIMAL);
        Label lblK2 = new Label("k2");
        fldK2 = new TextField();
        fldK2.setConstraint(TextField.DECIMAL);
        Label lblBE = new Label("ХЕ");
        fldBE = new TextField();
        fldBE.setConstraint(TextField.DECIMAL);
        fldK1.addActionListener(this);
        fldK2.addActionListener(this);
        fldBE.addActionListener(this);

        GroupLayout layout = new GroupLayout(con);
        con.setLayout(layout);
        
        layout.setHorizontalGroup(layout.createParallelGroup()
                .add(layout.createSequentialGroup()
                    .add(lblK1)
                    .add(1)
                    .add(fldK1, 0, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(3)
                    .add(lblK2)
                    .add(1)
                    .add(fldK2, 0, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .add(layout.createSequentialGroup()
                    .add(lblBE)
                    .add(2)
                    .add(fldBE, 0, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    );
        
        layout.setVerticalGroup(layout.createSequentialGroup()
                .add(layout.createParallelGroup()
                    .add(lblK1)
                    .add(fldK1)
                    .add(lblK2)
                    .add(fldK2))
                .add(layout.createParallelGroup()
                    .add(lblBE)
                    .add(fldBE)));
        
        return con;
    }

    public void pauseApp() {
    }

    public void destroyApp(boolean unconditional) {
        saveData(false);
        notifyDestroyed();
    }
    private String getAboutText() {
        return "Программа для быстрого подсчета доз.\n" +
                "Автор Топоров Константин (Connie), идея программы: " +
                "Лев Фонарёв, бета тестер -=eX3em'z=-\n" +
                "Сайт программы http://diacalc.org";
    }

    private void makeText(){
        Hashtable hs = new Hashtable();
        hs.put(new Integer('1'), "1+");//9
        hs.put(new Integer('2'), "2-");//2
        hs.put(new Integer('3'), "3");//3
        hs.put(new Integer('4'), "4*");//4
        hs.put(new Integer('5'), "5/");//5
        hs.put(new Integer('6'), "6");//6
        hs.put(new Integer('7'), "7(");//7
        hs.put(new Integer('8'), "8)");//8
        hs.put(new Integer('9'), "9");//9
        hs.put(new Integer('0'), "0");//9
        hs.put(new Integer('#'), ".");//9

        TextField.addInputMode("1+2", hs, false);

        TextField.setDefaultChangeInputModeKey(new Integer('*').intValue());
        TextField.setDefaultInputModeOrder(new String[] {"1+2"});
        TextField.setClearText("Очистить");
    }
    private Vector getFirstTagged(String output, String tag) throws Exception{
        Vector strs = new Vector();
        String tagfullst = "<"+tag+">";
        String tagfullend = "</"+tag+">";
        int st = 0;
        int end = 0;
        while (output.length()>0 && st>=0 && end>=0){
            st = output.indexOf(tagfullst);
            end = output.indexOf( tagfullend, st+tagfullst.length());
            if (st>=0 && end>=0){
                strs.addElement(output.substring(st+tagfullst.length(), end));
                output = output.substring(end+tagfullend.length());
            }
            else new Exception("no needed tag "+tag+" found");
        }
        return strs;
    }
    private Vector split(String st,String div){
        Vector res = new Vector();
        int start = 0;
        int pos = 0;
        while(pos<st.length()){
            pos = st.indexOf(div, start);
            if (pos>=0){
                res.addElement(st.substring(start, pos));
                start = pos + div.length();
            }
            else break;
        }
        res.addElement(st.substring(start, st.length()));
        return res;
    }
    private Vector getProducts(String st){
        Vector arr = new Vector();
        Vector lines = split(st.trim(),"\n");
        for(int i=0;i<lines.size();i++){
          if (((String)lines.elementAt(i)).trim().length()>0){
            Vector param = split( ((String)lines.elementAt(i)).trim(),
                    " ");
            int gi;
            float c,f,p;
            if (param.size()>4){
                try{
                    gi = Integer.parseInt((String)param.elementAt(param.size()-1));
                    c = Float.parseFloat( 
                    ((String)param.elementAt(param.size()-2)).replace(',', '.') );
                    f = Float.parseFloat(
                    ((String)param.elementAt(param.size()-3)).replace(',', '.') );
                    p = Float.parseFloat(
                    ((String)param.elementAt(param.size()-4)).replace(',', '.'));
                }catch(Exception ex){
                    //ex.printStackTrace();
                    gi = 0;
                    c = f = p = 0f;
                }
                String name ="";
                for(int j=0;j<(param.size()-4);j++){
                    name += (String)param.elementAt(j) + " ";
                }
                name = name.trim();
                arr.addElement(new ProductW(name,p,f,c,gi,100f));
            }
          }
        }
        return arr;
    }
    public void loadProductFromFile(String file){
        //теперь разбиваем на тэги и создаем форму заново
        String text = null;
        try{
            FileConnection fc = (FileConnection) Connector.open(file,Connector.READ);
            InputStreamReader is = new InputStreamReader(fc.openInputStream(),"UTF8");

            StringBuffer sb = new StringBuffer();
            int chr = 0;
            while ((chr = is.read()) != -1)
                sb.append((char) chr);
            is.close();
            fc.close();
            text = sb.toString();
        }catch(Exception ex){
            Dialog.show("Ошибка",
                new TextArea("Произошла ошибка при попытке чтения файла\n"+
                    ex.getMessage(),3,20),
                new Command [] {new Command("Ok")},
                Dialog.TYPE_ERROR, null);
        }
        if (text!=null && text.length()>0){
            int sz = 0;
            Vector groups = new Vector();
            text = text.replace('\r', ' ');
            try{
                 groups = getFirstTagged(text,"meal");
                 sz = groups.size();
            }catch(Exception ex){}
            if (sz>0){
                Vector [] base_tmp = new Vector[sz];//Сколько групп получилось

                //наполняем
                for(int i=0;i<sz;i++){
                    base_tmp[i] = getProducts( (String)groups.elementAt(i) );
                }
                //Продукты получились
                this.base = base_tmp;
                saveData(true);
                GROUPS = base_tmp.length;
                models = new DefaultListModel[GROUPS];
                weights = new TextField[GROUPS];
                System.gc();
                Display.getInstance().callSerially(new Runnable() {
                    public void run() {
                        setMainForm();
                    }
                });
            }
        }
    }
    
    private static class TickerRenderer extends DefaultListCellRenderer {
      private AnimatedRenderer selectedRenderer = new AnimatedRenderer(false);
      private List parentList;
      private Container cont = new Container(new BoxLayout(BoxLayout.Y_AXIS));
      private AnimatedRenderer descr = new AnimatedRenderer(false);

      public TickerRenderer() {
          super(false);
          selectedRenderer.setTickerEnabled(true);
          descr.setTickerEnabled(true);
          descr.getSelectedStyle().setBgTransparency(0);
          descr.setFocus(true);

          cont.addComponent(selectedRenderer);
          cont.addComponent(descr);
      }
      public void restartTicker(){
          descr.resetPosition();
          selectedRenderer.resetPosition();
      }
      public boolean animate() {
          if(parentList != null && parentList.getComponentForm() != null) {
              if(selectedRenderer.isTickerRunning()) {
                  if(selectedRenderer.animate()) {
                      parentList.repaint();
                  }
              }
              if(descr.isTickerRunning()) {
                  if(descr.animate()) {
                      parentList.repaint();
                  }
              }
          }
          return super.animate();
      }

      public Component getListCellRendererComponent(List list, Object value, int index, boolean isSelected) {
          ProductW pr = (ProductW)value;
          if(isSelected) {
              selectedRenderer.getListCellRendererComponent(list, pr.getName(), index, isSelected);
              descr.getListCellRendererComponent(list,
                        "Б:"+pr.getProt()+
                        " Ж:"+pr.getFat() +
                        " У:"+pr.getCarb() +
                        " ГИ:"+pr.getGi(),
                      index, isSelected);
                      //.setText();
              // sometimes the list asks for a dummy selected value for size calculations and this might
              // break the tickering state
              if(index == list.getSelectedIndex()) {
                  if(selectedRenderer.shouldTickerStart()) {
                      if(!selectedRenderer.isTickerRunning()) {
                          parentList = list;
                          list.getComponentForm().registerAnimated(this);
                          selectedRenderer.startTicker(UIManager.getInstance().getLookAndFeel().getTickerSpeed(), true);
                      }
                  } else {
                      if(selectedRenderer.isTickerRunning()) {
                          selectedRenderer.stopTicker();
                      }
                  }
                  if (descr.shouldTickerStart()){
                      if (!descr.isTickerRunning()){
                          parentList = list;
                          list.getComponentForm().registerAnimated(this);
                          descr.startTicker(UIManager.getInstance().getLookAndFeel().getTickerSpeed(), true);
                      }
                  }
                  else{
                      if(descr.isTickerRunning()) {
                          descr.stopTicker();
                      }
                  }
              }
              return cont;
          } else {
              return super.getListCellRendererComponent(list, value, index, isSelected);
          }
      }
  }

}
