/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Toporov Konstantin.
 */

package org.diacalc.simple;

/**
 *
 * @author Toporov Konstantin <www.diacalc.org>
 */
import com.sun.lwuit.events.*;
import com.sun.lwuit.*;
import com.sun.lwuit.layouts.BorderLayout;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;
import com.sun.lwuit.tree.*;
import javax.microedition.io.Connector;
import javax.microedition.io.file.FileConnection;
import javax.microedition.io.file.FileSystemRegistry;

public class ExploreForm extends Form implements ActionListener{
    private Form owner;
    Tree tree;

    private final static int BACK_COMMAND = 1;
    private final static int HELP_COMMAND = 2;

    private final static Command backCommand = new Command("Назад",BACK_COMMAND);
    private final static Command helpCommand = new Command("Помощь",HELP_COMMAND);

    public ExploreForm(String title, 
            Form owner,
            final DCJsimple midlet) throws Exception{
        super(title);
        this.owner = owner;
        setLayout(new BorderLayout());
        setScrollable(false);

        Image folder = null;
        Image folderOp = null;
        Image fileIc = null;
        try{
            folder = Image.createImage("/folder.png");
            folderOp = Image.createImage("/folderopen.png");
            fileIc = Image.createImage("/file.png");
        }catch (IOException ex){}

        if (folder!=null)   Tree.setFolderIcon(folder);
        if (folderOp!=null) Tree.setFolderOpenIcon(folderOp);
        if (fileIc!=null)   Tree.setNodeIcon(fileIc);
        tree = new Tree(new FileTreeModel()){
                private String buf;
                protected String childToDisplayLabel(Object child) {
                    if (((String) child).endsWith("/")) {
                        //Каталог
                        buf = ((String) child).substring(1, ((String) child).length()-1 );
                        return buf.substring(buf.lastIndexOf('/')+1, buf.length());
                        //return ((String) child).substring(((String) child).lastIndexOf('/', ((String) child).length() - 2)+1);
                    }
                    return ((String) child).substring(((String) child).lastIndexOf('/')+1);
                }
            };
        tree.addLeafListener(new ActionListener(){
            public void actionPerformed(ActionEvent ev){
                midlet.loadProductFromFile((String)tree.getSelectedItem());
            }
        });
        addComponent(BorderLayout.CENTER,tree);

        addCommand(backCommand);
        addCommand(helpCommand);
        setBackCommand(backCommand);
        addCommandListener(this);
    }
    public void actionPerformed(ActionEvent ev){
        switch (ev.getCommand().getId()){
            case BACK_COMMAND:  ExploreForm.this.owner.show();
                                break;
            case HELP_COMMAND: Form helpForm = new Form("Импорт помощь");
                        helpForm.setScrollable(false);
                        helpForm.setLayout(new BorderLayout());
                        TextArea helpText = new TextArea(getHelpText(), 5, 10);
                        helpText.setEditable(false);
                        helpForm.addComponent(BorderLayout.CENTER, helpText);
                        Command bkCm = new Command("Назад"){
                            public void actionPerformed(ActionEvent evt) {
                                ExploreForm.this.show();
                            }
                        };
                        helpForm.addCommand(bkCm);
                        helpForm.setBackCommand(bkCm);
                        helpForm.show();
                                break;
        }
        
    }
    private String getHelpText(){
        return "Для импорта базы продуктов необходимо выбрать, " +
                "предварительно закачанный в телефон файл meal.txt\n" +
                "О формате этого файла читайте на сайте diacalc.org\n" +
                "При просмотре фаловой системы телефона отображаются только " +
                "папки и файлы с именем meal.txt.";
    }
    public class FileTreeModel implements TreeModel {
        public FileTreeModel() throws Exception{
            try{
                Enumeration e = FileSystemRegistry.listRoots();
            } catch(Throwable err) {
                throw new Exception("Program don't have access to file system");
            }
        }


        public Vector getChildren(Object parent) {
            Vector response = new Vector();

            try {
                if(parent == null) {
                    Enumeration e = FileSystemRegistry.listRoots();
                    while(e.hasMoreElements()) {
                        String nx = (String)e.nextElement();
                        if (nx.endsWith(DCJsimple.MEALFILE) || nx.endsWith("/"))
                            response.addElement("file:///" + nx);
                    }
                } else {
                    String name = (String)parent;
                    FileConnection c = (FileConnection)Connector.open(name, Connector.READ);
                    Enumeration e = c.list();
                    while(e.hasMoreElements()) {
                        String nx = (String)e.nextElement();
                        if (nx.endsWith(DCJsimple.MEALFILE) || nx.endsWith("/"))
                            response.addElement(name + nx);
                    }
                    c.close();
                }
            } catch(Throwable err) {
                err.printStackTrace();
                return getChildren(parent);
            }
            return response;
        }

        public boolean isLeaf(Object node) {
            boolean d = true;
            try {
                FileConnection c = (FileConnection)Connector.open((String)node, Connector.READ);
                d = c.isDirectory();
                c.close();
            } catch(Throwable err) {
                err.printStackTrace();
                return isLeaf(node);
            }
            return !d;
        }
    }
}
